--[[

$Revision: 60141 $

(C) Copyright 2007,2008 Bethink (bethink@naef.com)

This file is part of CrossRealm.

CrossRealm is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CrossRealm is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CrossRealm. If not, see <http://www.gnu.org/licenses/>.

]]


----------------------------------------------------------------------
-- Acquire libraries and declare add-on

local L = AceLibrary("AceLocale-2.2"):new("CrossRealm")
CrossRealm = AceLibrary("AceAddon-2.0"):new(
	"AceConsole-2.0",
	"AceDB-2.0",
	"FuBarPlugin-2.0")


----------------------------------------------------------------------
-- Constants

-- Quality map
local QUALITY_MAP = {
	[0] = L["POOR"],
	[1] = L["COMMON"],
	[2] = L["UNCOMMON"],
	[3] = L["RARE"],
	[4] = L["EPIC"]
}

local INVERSE_QUALITY_MAP = { }
for key, value in pairs(QUALITY_MAP) do
	INVERSE_QUALITY_MAP[value] = key
end

-- Ace options
local ACE_OPTS = {
	type = "group",
	args = {
		minProfit = {
			type = "text",
			name = L["MIN_PROFIT"],
			desc = L["MIN_PROFIT_DESC"],
			usage = L["MIN_PROFIT_USAGE"],
			get = "GetMinProfit",
			set = "SetMinProfit",
			validate = "ValidateMinProfit"
		},
		minProfitPct = {
			type = "text",
			name = L["MIN_PROFIT_PCT"],
			desc = L["MIN_PROFIT_PCT_DESC"],
			usage = L["MIN_PROFIT_PCT_USAGE"],
			get = "GetMinProfitPct",
			set = "SetMinProfitPct",
			validate = "ValidateMinProfitPct"
		},
		minQuality = {
			type = "text",
			name = L["MIN_QUALITY"],
			desc = L["MIN_QUALITY_DESC"],
			get = "GetMinQuality",
			set = "SetMinQuality",
			validate = {
				L["POOR"],
				L["COMMON"],
				L["UNCOMMON"],
				L["RARE"],
				L["EPIC"]
			}
		},
		wowecon = {
			type = "execute",
			name = L["WOWECON"],
			desc = L["WOWECON_DESC"],
			func = "ReportWowecon"
		},
		opposite = {
			type = "execute",
			name = L["OPPOSITE"],
			desc = L["OPPOSITE_DESC"],
			func = "ReportOpposite"
		}
	}
}


----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function CrossRealm:OnInitialize ()
	-- Register chat commands
	self:RegisterChatCommand(L["SLASH_COMMANDS"], ACE_OPTS)
	
	-- Register database
	self:RegisterDB("CrossRealmDB", "CrossRealmDBPerChar")
	self:RegisterDefaults("profile", {
		minProfit = 10000, -- 1 gold
		minProfitPct = 20, -- 20 percent
		minQuality = 3 -- rare
	})

	-- FuBar
	self.OnMenuRequest = ACE_OPTS
	self.hasIcon = true
	self:SetIcon("Interface\\Icons\\INV_Ore_Gold_01.blp")
end


----------------------------------------------------------------------
-- Settings

-- Returns the minimum profit
function CrossRealm:GetMinProfit ()
	return self.db.profile.minProfit
end

-- Sets the minimum profit
function CrossRealm:SetMinProfit (minProfit)
	self.db.profile.minProfit = tonumber(minProfit)
end

-- Validates the minimum profit
function CrossRealm:ValidateMinProfit (minProfit)
	local number = tonumber(minProfit)
	return number ~= nil and number >= 0
end

-- Returns the minimum profit in percent
function CrossRealm:GetMinProfitPct ()
	return self.db.profile.minProfitPct
end

-- Sets the minimum profit in percent
function CrossRealm:SetMinProfitPct (minProfitPct)
	self.db.profile.minProfitPct = tonumber(minProfitPct)
end

-- Validates the minimum profit in percent
function CrossRealm:ValidateMinProfitPct (minProfitPct)
	local number = tonumber(minProfitPct)
	return number ~= nil and number >= 0 and number <= 100
end

-- Returns the minimum quality
function CrossRealm:GetMinQuality ()
	return QUALITY_MAP[self.db.profile.minQuality]
end

-- Sets the minimum quality
function CrossRealm:SetMinQuality (minQuality)
	self.db.profile.minQuality = INVERSE_QUALITY_MAP[minQuality]
end


----------------------------------------------------------------------
-- Reporting

-- Generates a report using WOWEcon.com as the reference price source
function CrossRealm:ReportWowecon ()
	-- Check is WOWEcon_PriceMod is available
	if not IsAddOnLoaded("WOWEcon_PriceMod") then
		DEFAULT_CHAT_FRAME:AddMessage(L["WOWECON_PRICEMOD_NOT_LOADED"])
		return
	end

	-- Load auctioneer as needed
	if not IsAddOnLoaded("Auctioneer") then
		LoadAddOn("Auctioneer")
	end

	-- Get a snapshot and process
	local auctions = Auctioneer.SnapshotDB.Query(nil, nil, CrossRealm.AuctioneerFilter, nil)
	for _, auction in ipairs(auctions) do
		local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
		local itemLink = Auctioneer.ItemDB.GetItemLink(itemKey)
		local woweconPrice = Wowecon.API.GetAuctionPrice_ByLink(
				itemLink,
				Wowecon.API.GLOBAL_PRICE)
		if woweconPrice then
			woweconPrice = woweconPrice * auction.count
			local buyoutPrice = auction.buyoutPrice
			local profit = woweconPrice - buyoutPrice
			local profitPct = profit / buyoutPrice * 100
			if profit >= self.db.profile.minProfit and
					profitPct >= self.db.profile.minProfitPct then
				DEFAULT_CHAT_FRAME:AddMessage(string.format(
					L["REPORT_LINE"],
					itemLink,
					auction.count,
					buyoutPrice / 10000,
					woweconPrice / 10000,
					profit / 10000,
					profitPct))
			end
		end
	end
end

-- Generates a report using the opposite faction as the reference price source
function CrossRealm:ReportOpposite ()
	-- Load auctioneer as needed
	if not IsAddOnLoaded("Auctioneer") then
		LoadAddOn("Auctioneer")
	end
	
	-- Get the opposite AH key
	local oppositeKey = Auctioneer.Util.GetOppositeKey()
	
	-- Get a snapshot and process
	local auctions = Auctioneer.SnapshotDB.Query(nil, nil, CrossRealm.AuctioneerFilter, nil)
	for _, auction in ipairs(auctions) do
		local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
		local oppositeMedian = Auctioneer.Statistic.GetUsableMedian(itemKey, oppositeKey)
		if oppositeMedian then
			oppositeMedian = oppositeMedian * auction.count
			local buyoutPrice = auction.buyoutPrice
			local profit = oppositeMedian - buyoutPrice
			local profitPct = profit / buyoutPrice * 100
			if profit >= self.db.profile.minProfit and
					profitPct >= self.db.profile.minProfitPct then
				DEFAULT_CHAT_FRAME:AddMessage(string.format(
					L["REPORT_LINE"],
					Auctioneer.ItemDB.GetItemLink(itemKey),
					auction.count,
					buyoutPrice / 10000,
					oppositeMedian / 10000,
					profit / 10000,
					profitPct))
			end
		end
	end
end

-- Filters auctions from Auctioneer
function CrossRealm.AuctioneerFilter (auction, filterArgs)
	-- Check quality
	local itemKey = Auctioneer.ItemDB.CreateItemKeyFromAuction(auction)
	local itemInfo = Auctioneer.ItemDB.GetItemInfo(itemKey)
	if itemInfo.quality < CrossRealm.db.profile.minQuality then
		return false
	end

	-- Check for buyout auction
	if auction.buyoutPrice == 0 then
		return false
	end

	-- Passed
	return true
end




